package org.eclipse.jdt.internal.compiler.apt.dispatch;

package .tmp;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import javax.tools.ForwardingJavaFileObject;
import javax.tools.JavaFileObject;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BatchFilerImpl;
import org.eclipse.jdt.internal.compiler.batch.CompilationUnit;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFileReader;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFormatException;
import org.eclipse.jdt.internal.compiler.env.IBinaryType;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.lookup.BinaryTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;

public class HookedJavaFileObject extends ForwardingJavaFileObject<JavaFileObject> {
  protected final BatchFilerImpl _filer;
  
  protected final String _fileName;
  
  private boolean _closed = false;
  
  private String _typeName;
  
  public HookedJavaFileObject(JavaFileObject fileObject, String fileName, String typeName, BatchFilerImpl filer) {
    super(fileObject);
    this._filer = filer;
    this._fileName = fileName;
    this._typeName = typeName;
  }
  
  public OutputStream openOutputStream() throws IOException {
    return (OutputStream)new ForwardingOutputStream(this, super.openOutputStream());
  }
  
  public Writer openWriter() throws IOException {
    return (Writer)new ForwardingWriter(this, super.openWriter());
  }
  
  protected void closed() {
    if (!this._closed) {
      CompilationUnit unit;
      IBinaryType binaryType;
      ClassFileReader classFileReader;
      this._closed = true;
      switch (getKind()) {
        case SOURCE:
          unit = new CompilationUnit(null, this._fileName, null);
          this._filer.addNewUnit((ICompilationUnit)unit);
          break;
        case null:
          binaryType = null;
          try {
            classFileReader = ClassFileReader.read(this._fileName);
          } catch (ClassFormatException classFormatException) {
            ReferenceBinding type = this._filer._env._compiler.lookupEnvironment.getType(CharOperation.splitOn('.', this._typeName.toCharArray()));
            if (type != null)
              this._filer.addNewClassFile(type); 
          } catch (IOException iOException) {}
          if (classFileReader != null) {
            char[] name = classFileReader.getName();
            ReferenceBinding type = this._filer._env._compiler.lookupEnvironment.getType(CharOperation.splitOn('/', name));
            if (type != null && type.isValidBinding()) {
              if (type.isBinaryBinding()) {
                this._filer.addNewClassFile(type);
                break;
              } 
              BinaryTypeBinding binaryBinding = new BinaryTypeBinding(type.getPackage(), (IBinaryType)classFileReader, this._filer._env._compiler.lookupEnvironment, true);
              if (binaryBinding != null)
                this._filer.addNewClassFile((ReferenceBinding)binaryBinding); 
            } 
          } 
          break;
      } 
    } 
  }
}
